-- ************************************************************
-- *		 (C) Copyright 2012, Offete Enterprises, Inc.	  *
-- *					 ALL RIGHTS RESERVED				  *
-- *==========================================================*
-- * Project:			 FG in PROASIC					   	  *
-- * File:				 ep16_chip.vhd						  *
-- * Author:			 Chien-Chia Wu						  *
-- * Description:		 Top level block					  *
-- *														  *
-- * Hierarchy:parent:										  *
-- *		   child :										  *  
-- *														  *
-- * Revision History:										  *
-- * Date		 By Who		  Modification					  *
-- * 09/19/02	 Chien-Chia Wu   Branch from ep16a.			  *
-- * 01/02/03	 Chien-Chia Wu   Add SDI.					  *
-- * 01/29/03	 Chien-Chia Wu   Add Boot.					  *
-- * 02/24/03	 Chien-Chia Wu   Modify the module as 32-bits * 
-- *							  version.					  * 
-- * 02/27/03	 Chien-Chia Wu   Modify SDRAM byte-assecable  *
-- * 03/02/03	 Chien-Chia Wu   Add internal SRAM module.    *
-- * 06/29/06	 Chen-Hanson Ting Add HMPP/Shifter/Controller *
-- * 11/18/10	 Chen-Hanson Ting LatticeXP2 Brevia Kit   	  *
-- * 02/29/12	 Chen-Hanson Ting Back to eP16			   	  *
-- * 05/23/16	 Chen-Hanson PDP1            			   	  *
-- ************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;
use ieee.std_logic_unsigned.all;

entity pdp1_chip is 
	generic(width: integer := 18);
port(
	-- input port
	aclk:			in		std_logic; 
	arst:			in		std_logic;
	uart_i:			in		std_logic;
	-- output port
	uart_o:			out		std_logic;
	-- io port
	ioport:			inout 	std_logic_vector(0 to width-1)
	);
end entity pdp1_chip;


architecture behavioral of pdp1_chip is
  -- component declaration
  component pdp1 is 
	port(
		-- input port
		clr:		in	std_logic;
		clk:		in	std_logic;
		data_i:		in	std_logic_vector(0 to width-1);
		-- output port
		io_read:	out	std_logic;
		io_write:	out	std_logic;
		mem_read:	out	std_logic;
		mem_write:	out	std_logic;
		addr_o:		out	std_logic_vector(0 to 11);
		data_o:		out	std_logic_vector(0 to width-1)
	);
  end component;

component ram_memory
	port (
		Clock: 		in  	std_logic;
		ClockEn: 	in  	std_logic; 
		Reset: 		in  	std_logic;
		WE: 		in  	std_logic; 
		Address: 	in  	std_logic_vector(0 to 11); 
		Data: 		in  	std_logic_vector(0 to width-1); 
		Q: 			out  	std_logic_vector(0 to width-1)
	);
end component;

  component uart is
	port(
		-- input
		clk_i:		in		std_logic;
		rst_i:		in		std_logic;
		ce_i:		in		std_logic;
		read_i:		in		std_logic;
		write_i:	in		std_logic;
		addr_i:		in		std_logic_vector(1 downto 0);
		data_i:		in		std_logic_vector(15 downto 0);
		-- output
		data_o:		out		std_logic_vector(15 downto 0);
		rx_empty_o: out		std_logic;
		rx_irq_o:   out		std_logic;
		tx_irq_o:   out		std_logic;
		-- external interface
		rxd_i:		in		std_logic;
		txd_o:		out		std_logic;
		cts_i:		in		std_logic;
		rts_o:		out		std_logic
	);
  end component;

component gpio
	port(
		-- input port
		clr: 		in		std_logic;
		clk: 		in		std_logic;
		write: 		in		std_logic;
		read: 		in		std_logic;
		ce: 		in		std_logic;
		addr: 		in		std_logic_vector(0 to 1);
		data_in: 	in		std_logic_vector(0 to width-1);
		gpio_in: 	in		std_logic_vector(0 to width-1); 
		-- output port
		data_out: 	out		std_logic_vector(0 to width-1);
		gpio_out: 	out		std_logic_vector(0 to width-1);
		gpio_dir: 	out		std_logic_vector(0 to width-1)
	);
end component;

-- internal signal for cpu
	signal cpu_data_i:		std_logic_vector(0 to width-1);
	signal cpu_addr_o:		std_logic_vector(0 to 11);
	signal cpu_data_o:		std_logic_vector(0 to width-1);
	signal cpu_m_read:		std_logic;
	signal cpu_m_write:		std_logic;
	signal cpu_io_read:		std_logic;
	signal cpu_io_write:	std_logic;
 
-- interal signal for memory
	signal m_rst:			std_logic;
	signal m_clk:			std_logic;
	signal bclk:			std_logic;
	signal memory_data_o:	std_logic_vector(0 to width-1);

-- internal signal for uart
	signal uart_ce:			std_logic; 
	signal uart_addr:		std_logic_vector(0 to 1);
--	signal uart_data_i:		std_logic_vector(0 to width-1);
	signal uart_data_o:		std_logic_vector(0 to width-1);
	signal uart_rx_empty:	std_logic;
	signal uart_rx_irq:		std_logic;
	signal uart_tx_irq:		std_logic;
	signal uart_rxd:		std_logic;
	signal uart_txd:		std_logic;
	signal uart_cts:		std_logic;
	signal uart_rts:		std_logic;

-- internal signal for gpio
	signal gpio_ce:			std_logic;
	signal gpio_addr:		std_logic_vector(0 to 1);
	signal gpio_data_i:		std_logic_vector(0 to width-1);
	signal gpio_in:			std_logic_vector(0 to width-1); 
	signal gpio_data_o:		std_logic_vector(0 to width-1);
	signal gpio_out:		std_logic_vector(0 to width-1);
	signal gpio_dir:		std_logic_vector(0 to width-1);
 
begin
-- ************************************************************   
--			Component Binding
-- ************************************************************   
-- ========================= CPU Block ========================	
	cpu1: pdp1 
		port map (
		-- input port
		clk 		=> bclk,
		clr 		=> m_rst,
		data_i 		=> cpu_data_i,
		io_read 	=> cpu_io_read,
		io_write 	=> cpu_io_write,
		mem_read 	=> cpu_m_read,
		mem_write 	=> cpu_m_write,
		addr_o 		=> cpu_addr_o,
		data_o 		=> cpu_data_o
		);

-- ************************************************************   
--			Internal Globle Signal Circuit
-- ************************************************************   

	m_rst <= not arst;
	m_clk <= not bclk;

	cpu_data_i  <=  
		memory_data_o when(cpu_io_read = '0')
		else
		uart_data_o when ((cpu_addr_o(0 to 9)="0000000001") and (cpu_io_read = '1'))
		else
		gpio_data_o when ((cpu_addr_o(0 to 9)="0000000000") and (cpu_io_read = '1'))
		else (others => 'Z');

-- ========================= RAM Block ========================	
ram_memory_0 : ram_memory
	PORT MAP (
		Address	=> cpu_addr_o,
		Clock	=> m_clk,
		ClockEn	=> '1',
		Reset	=> '0',
		Data	=> cpu_data_o,
		WE		=> cpu_m_write,
		Q		=> memory_data_o
	);

-- ========================= UART Block =======================	
  uart1 : uart
	port map (
		-- input
		clk_i 	=> bclk,
		rst_i 	=> m_rst,
		ce_i 	=> uart_ce,
		read_i 	=> cpu_io_read,
		write_i => cpu_io_write,
		addr_i 	=> uart_addr,
		data_i 	=> cpu_data_o(2 to 17),
		-- output
		data_o 	=> uart_data_o(2 to 17),
		rx_empty_o => uart_rx_empty,
		rx_irq_o => uart_rx_irq,
		tx_irq_o => uart_tx_irq,
		-- external interface
		rxd_i 	=> uart_i,
		txd_o 	=> uart_o,
		cts_i => uart_cts,
		rts_o => uart_rts
	);
	uart_ce <= '1' when (cpu_addr_o(0 to 9)="0000000001")
  		else '0';
	uart_addr <= cpu_addr_o(10 to 11);
	uart_data_o(0 to 1) <= "00";
	uart_cts <= '1';
 
 
-- ========================= GPIO Block =======================	
  gpio1 : gpio
   port map (
		-- input port
		clr => m_rst,
		clk => bclk,
		write => cpu_io_write,
		read => cpu_io_read,
		ce => gpio_ce,
		addr => gpio_addr,
		data_in => cpu_data_o,
		gpio_in => ioport, 
		-- output port
		data_out => gpio_data_o,
		gpio_dir => gpio_dir,
		gpio_out => gpio_out
	);
	gpio_ce <= '1' when (cpu_addr_o(0 to 9)="0000000000")
		else '0';
	gpio_addr <= cpu_addr_o(10 to 11);
	ioport(0)  <= gpio_out(0)  when gpio_dir(0)='1'  else 'Z';
	ioport(1)  <= gpio_out(1)  when gpio_dir(1)='1'  else 'Z';
	ioport(2)  <= gpio_out(2)  when gpio_dir(2)='1'  else 'Z';
	ioport(3)  <= gpio_out(3)  when gpio_dir(3)='1'  else 'Z';
	ioport(4)  <= gpio_out(4)  when gpio_dir(4)='1'  else 'Z';
	ioport(5)  <= gpio_out(5)  when gpio_dir(5)='1'  else 'Z';
	ioport(6)  <= gpio_out(6)  when gpio_dir(6)='1'  else 'Z';
	ioport(7)  <= gpio_out(7)  when gpio_dir(7)='1'  else 'Z';
	ioport(8)  <= gpio_out(8)  when gpio_dir(8)='1'  else 'Z';
	ioport(9)  <= gpio_out(9)  when gpio_dir(9)='1'  else 'Z';
	ioport(10) <= gpio_out(10) when gpio_dir(10)='1' else 'Z';
	ioport(11) <= gpio_out(11) when gpio_dir(11)='1' else 'Z';
	ioport(12) <= gpio_out(12) when gpio_dir(12)='1' else 'Z';
	ioport(13) <= gpio_out(13) when gpio_dir(13)='1' else 'Z';
	ioport(14) <= gpio_out(14) when gpio_dir(14)='1' else 'Z';
	ioport(15) <= gpio_out(15) when gpio_dir(15)='1' else 'Z';
	ioport(16) <= gpio_out(16) when gpio_dir(16)='1' else 'Z';
	ioport(17) <= gpio_out(17) when gpio_dir(17)='1' else 'Z';

-- **********************************************************	
--		Reduce clock rate 
-- **********************************************************	
	clocking:
	process(m_rst, aclk)
	begin
	if (m_rst='1') then
		bclk <= '1';
	elsif ( aclk'event and aclk='1') then
		bclk <= not bclk;
	end if;  
	end process clocking;

  end behavioral;
